//--------------------------------------------------------------------------
//
//   Copyright (c) 2002, Colin Granville
//
//   All rights reserved.
//
//   Redistribution and use in source and binary forms, with or
//   without modification, are permitted provided that the following 
//   conditions are met:
//
//      * Redistributions of source code must retain the above copyright 
//        notice, this list of conditions and the following disclaimer.
//
//      * Redistributions in binary form must reproduce the above 
//        copyright notice, this list of conditions and the following 
//        disclaimer in the documentation and/or other materials 
//        provided with the distribution.
//
//      * The name Colin Granville may not be used to endorse or promote 
//        products derived from this software without specific prior 
//        written permission.
//
//   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
//   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
//   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
//   FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
//   COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
//   INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
//   (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
//   SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
//   HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
//   STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
//   OF THE POSSIBILITY OF SUCH DAMAGE.
//
//--------------------------------------------------------------------------

#ifndef GuiFlexDA_h
#define GuiFlexDA_h

#include "GuiFlex.h"
#include "GuiAtExit.h"

class GuiFlexDA : public GuiFlex, private GuiAtExit
{
  public:
    GuiFlexDA(const char *name, int *error_fd, int dynamic_size);
    virtual         ~GuiFlexDA();
    virtual bool    alloc(GuiFlexAnchor&,size_t n);
    virtual bool    free(GuiFlexAnchor&);
    virtual size_t  size(const GuiFlexAnchor&) const;
    virtual bool    extend(GuiFlexAnchor&,size_t new_size);
    virtual bool    midExtend(GuiFlexAnchor&,size_t at,int by);
    virtual bool    reanchor(GuiFlexAnchor& to, GuiFlexAnchor& from);
    virtual bool    setBudge(bool on);
    virtual void    saveHeapInfo(char* filename); 
    virtual bool    compact();
    virtual bool    setDeferredCompaction(bool on);
    virtual size_t  availableMemory();
    
    bool            isOk() const    {return base != 0;}

  private:

    struct Object
    {
      size_t         size;
      GuiFlexAnchor* anchor;

      size_t         capacity()     {return ((size+3)&~3);}
    };

    char*           base;
    int             number; 
    size_t          top;
    size_t          end;
    size_t          maxSize;

    virtual void    atExit();
    bool            resize(size_t new_size);
    Object*         getObject(const GuiFlexAnchor&) const;
    static          void setAnchor(GuiFlexAnchor&,Object*);
    void            reanchor(Object* from);
    Object*         topObject();
    Object*         nextObject(Object*);
};

#endif
