//--------------------------------------------------------------------------
//
//   Copyright (c) 2002, Colin Granville
//
//   All rights reserved.
//
//   Redistribution and use in source and binary forms, with or
//   without modification, are permitted provided that the following
//   conditions are met:
//
//      * Redistributions of source code must retain the above copyright
//        notice, this list of conditions and the following disclaimer.
//
//      * Redistributions in binary form must reproduce the above
//        copyright notice, this list of conditions and the following
//        disclaimer in the documentation and/or other materials
//        provided with the distribution.
//
//      * The name Colin Granville may not be used to endorse or promote
//        products derived from this software without specific prior
//        written permission.
//
//   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
//   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
//   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
//   FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
//   COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
//   INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
//   (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
//   SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
//   HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
//   STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
//   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
//   OF THE POSSIBILITY OF SUCH DAMAGE.
//
//--------------------------------------------------------------------------

#ifndef GuiTargets_h
#define GuiTargets_h

#include "GuiRegister.h"

#define GUI_DECLARE_EVENT_TARGETS(TYPE)\
\
class GuiNullTarget : public GuiNullEventTarget\
{\
  public:\
    typedef void (TYPE::* Method)();\
    GuiNullTarget() : object(0),method(0)                 {}\
    GuiNullTarget(TYPE* ob,Method m,int interval=0)\
      : GuiNullEventTarget(interval),object(ob),method(m) {}\
    virtual void execute()                       {if (method) (object->*method)();}\
    void         setMethod(TYPE* ob,Method m) {object=ob;method=m;}\
    void         operator()(TYPE* ob,Method m,unsigned int interval=0)\
                  {setMethod(ob,m);setInterval(interval);}\
\
  private:\
    TYPE*   object;\
    Method method;\
};\
\
class GuiToolboxTarget : public GuiToolboxEventTarget\
{\
  public:\
    typedef Claim (TYPE::* Method)(GuiToolboxEvent&,const GuiIdBlock&);\
    GuiToolboxTarget() : object(0),method(0)                     {}\
    GuiToolboxTarget(GuiObject* o,int event_code,TYPE* ob,Method m)\
      : GuiToolboxEventTarget(o,event_code),object(ob),method(m) {}\
    virtual Claim execute(GuiToolboxEvent& t,const GuiIdBlock& i)\
                         {return (method)?(object->*method)(t,i):DONT_CLAIM;}\
    void          setMethod(TYPE* ob,Method m) {object=ob;method=m;}\
    void          operator()(const GuiObject* o,int event_code,TYPE* ob,Method m)\
                  {setListener(o,event_code);setMethod(ob,m);}\
\
  private:\
    TYPE*   object;\
    Method    method;\
};\
\
class GuiWimpTarget : public GuiWimpEventTarget\
{\
  public:\
    typedef Claim (TYPE::* Method)(GuiWimpPollBlock&,const GuiIdBlock&);\
    GuiWimpTarget() : object(0),method(0)                     {}\
    GuiWimpTarget(GuiObject* o,int event_code,TYPE* ob,Method m)\
      : GuiWimpEventTarget(o,event_code),object(ob),method(m) {}\
    virtual Claim execute(GuiWimpPollBlock& wpb,const GuiIdBlock& i)\
                         {return (method)?(object->*method)(wpb,i):DONT_CLAIM;}\
    void          setMethod(TYPE* ob,Method m) {object=ob;method=m;}\
    void          operator()(const GuiObject* o,int event_code,TYPE* ob,Method m)\
                  {setListener(o,event_code);setMethod(ob,m);}\
\
  private:\
    TYPE*   object;\
    Method    method;\
};\
\
class GuiMessageTarget : public GuiMessageEventTarget\
{\
  public:\
    typedef Claim (TYPE::* Method)(GuiWimpMessage&);\
    GuiMessageTarget() : object(0),method(0)                     {}\
    GuiMessageTarget(int message_code,TYPE* ob,Method m)\
      : GuiMessageEventTarget(message_code),object(ob),method(m) {}\
    virtual Claim execute(GuiWimpMessage& wm)        {return (method)?(object->*method)(wm):DONT_CLAIM;}\
    void          setMethod(TYPE* ob,Method m) {object=ob;method=m;}\
    void          operator()(int message_code,TYPE* ob,Method m)\
                  {setListener(message_code);setMethod(ob,m);}\
\
  private:\
    TYPE*   object;\
    Method    method;\
}

#endif
