//--------------------------------------------------------------------------
//
//   Copyright (c) 2002, Colin Granville
//
//   All rights reserved.
//
//   Redistribution and use in source and binary forms, with or
//   without modification, are permitted provided that the following 
//   conditions are met:
//
//      * Redistributions of source code must retain the above copyright 
//        notice, this list of conditions and the following disclaimer.
//
//      * Redistributions in binary form must reproduce the above 
//        copyright notice, this list of conditions and the following 
//        disclaimer in the documentation and/or other materials 
//        provided with the distribution.
//
//      * The name Colin Granville may not be used to endorse or promote 
//        products derived from this software without specific prior 
//        written permission.
//
//   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
//   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
//   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
//   FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
//   COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
//   INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
//   (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
//   SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
//   HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
//   STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
//   OF THE POSSIBILITY OF SUCH DAMAGE.
//
//--------------------------------------------------------------------------

#include "Bookmarks.h"
#include "GuiGadget.h"
#include "GuiTargets.h"
#include "DocViewToolbar.h"
#include "DocView.h"
#include "Document.h"
#include "UserEvents.h"

#define BASE_COMPONENT 0x80

Bookmarks::Bookmarks(DocView& v,DocViewToolbar& tb,GuiWindow& ancestor)
 : view(v),
   toolbar(tb),
   selectTarget(&ancestor,User_BookmarkSelect,this,(Claim (Bookmarks::*)(GuiToolboxEvent&, const GuiIdBlock&))&Bookmarks::select),
   adjustTarget(&ancestor,User_BookmarkAdjust,this,(Claim (Bookmarks::*)(GuiToolboxEvent&, const GuiIdBlock&))&Bookmarks::adjust)
{
  int i;
  for (i=0;i<4;i++) page[i].page=0;
}

//*************************************************************************
Claim Bookmarks::select(GuiToolboxEvent&,const GuiIdBlock& id_block)
{
  unsigned int num=id_block.self.component-BASE_COMPONENT;
  if (num<4 && page[num].page) view.setPage(page[num]);
  return CLAIM;
}

//*************************************************************************

Claim Bookmarks::adjust(GuiToolboxEvent&,const GuiIdBlock& id_block)
{
  unsigned int num=id_block.self.component-BASE_COMPONENT;
  if (num>=4) return CLAIM;
  int value=toolbar.getPage();
  GuiToolAction(toolbar,id_block.self.component).set(value!=0);
  page[num].page=value;
  page[num].docName=view.getDocument().getFileName();
  return CLAIM;
}
