//--------------------------------------------------------------------------
//
//   Copyright (c) 2002, Colin Granville
//
//   All rights reserved.
//
//   Redistribution and use in source and binary forms, with or
//   without modification, are permitted provided that the following 
//   conditions are met:
//
//      * Redistributions of source code must retain the above copyright 
//        notice, this list of conditions and the following disclaimer.
//
//      * Redistributions in binary form must reproduce the above 
//        copyright notice, this list of conditions and the following 
//        disclaimer in the documentation and/or other materials 
//        provided with the distribution.
//
//      * The name Colin Granville may not be used to endorse or promote 
//        products derived from this software without specific prior 
//        written permission.
//
//   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
//   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
//   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
//   FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
//   COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
//   INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
//   (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
//   SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
//   HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
//   STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
//   OF THE POSSIBILITY OF SUCH DAMAGE.
//
//--------------------------------------------------------------------------

#ifndef DocViewChoices_h
#define DocViewChoices_h

#include <string>

using namespace std;

class DocViewChoices
{
  public:
    enum {NO_IMAGES       = (1 << 1),
          NO_TEXT         = (1 << 2),
          NO_TYPE3_FONTS  = (1 << 3),
          NO_DRAWINGS     = (1 << 4),
          HIGHLIGHT_LINKS = (1 << 5)
         };

    DocViewChoices();
    ~DocViewChoices();

    bool    getNoImages() const     {return flags & NO_IMAGES ? 1:0;}
    void    setNoImages(int on)     {if (on) flags|=NO_IMAGES; else flags &= ~NO_IMAGES;}

    bool    getNoText() const       {return flags & NO_TEXT ? 1:0;}
    void    setNoText(int on)       {if (on) flags|=NO_TEXT; else flags &= ~NO_TEXT;}

    bool    getNoType3Fonts() const       {return flags & NO_TYPE3_FONTS ? 1:0;}
    void    setNoType3Fonts(int on)       {if (on) flags|=NO_TYPE3_FONTS; else flags &= ~NO_TYPE3_FONTS;}

    bool    getNoDrawings() const       {return flags & NO_DRAWINGS ? 1:0;}
    void    setNoDrawings(int on)       {if (on) flags|=NO_DRAWINGS; else flags &= ~NO_DRAWINGS;}

    bool    getHighlightLinks() const       {return flags & HIGHLIGHT_LINKS ? 1:0;}
    void    setHighlightLinks(int on)       {if (on) flags|=HIGHLIGHT_LINKS; else flags &= ~HIGHLIGHT_LINKS;}

    int     getScale() const        {return scale;}
    void    setScale(int s);

    int     getRotation() const     {return rotation;}
    void    setRotation(int r)      {rotation=r;}
      
    void    save();
  public:
    int         flags;
    int         scale;
    int         rotation;
};

#endif
