//--------------------------------------------------------------------------
//
//   Copyright (c) 2002, Colin Granville
//
//   All rights reserved.
//
//   Redistribution and use in source and binary forms, with or
//   without modification, are permitted provided that the following 
//   conditions are met:
//
//      * Redistributions of source code must retain the above copyright 
//        notice, this list of conditions and the following disclaimer.
//
//      * Redistributions in binary form must reproduce the above 
//        copyright notice, this list of conditions and the following 
//        disclaimer in the documentation and/or other materials 
//        provided with the distribution.
//
//      * The name Colin Granville may not be used to endorse or promote 
//        products derived from this software without specific prior 
//        written permission.
//
//   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
//   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
//   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
//   FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
//   COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
//   INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
//   (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
//   SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
//   HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
//   STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
//   OF THE POSSIBILITY OF SUCH DAMAGE.
//
//--------------------------------------------------------------------------

#ifndef DrawOutputFont_h
#define DrawOutputFont_h

#include "GuiDrawFileText.h"
#include "GuiAtExit.h"
#include "UnicodeToAcornMap.h"
#include "gtypes.h"
#include <string>

using namespace std;

class GfxState;
class GfxFont;

string toAcornLatin1(const char*, int len);

class DrawOutputFont
{
  public:
    DrawOutputFont* next;

    DrawOutputFont(GfxFont*);
    ~DrawOutputFont();
    const char*    getRiscOsFontName()     {return riscOsFontName;}
    GfxFont*       getFont()               {return font;}
    int            getFontIndex()          {return fontIndex;}
    unsigned int   getChar(unsigned int c) {return map->getChar(c);}
    double         getCharWidth(unsigned int c) {return charWidth[c];}

  private:
    const char*        riscOsFontName;
    GfxFont*           font;
    int                fontIndex;
    UnicodeToAcornMap* map;
    UnicodeToAcornMap* mapStore;
    double*            charWidth;

    void getSubstituteFont(const char* name);
    bool findFont(const char* name);
};   

class DrawOutputFontList : public GuiDrawFileTextState, public GuiAtExit
{
  public:
    DrawOutputFontList();
    ~DrawOutputFontList();
    void clear();

    void set(GfxState&);
    void setMatrix(GfxState&);
    void setCharMatrix(double fromCharWidth, double toCharWidth);
    unsigned int getChar(unsigned int c)      {return fontList->getChar(c);}
    double       getCharWidth(unsigned int c) {return fontList->getCharWidth(c);}
    int          getHandle()              {return handle;}

    const char*  getUsedFont(int index); //returns 0 if index >= last font used - index starts at 1;

    virtual void atExit();
    static  void declareFonts();
  private:
    DrawOutputFont* fontList;//top is the selected font
    double          charMatrix[4];
    double          matXScale;
    double          matYScale;

    int             handle;
    void            find(GfxState&);
    void            lose();
};

#endif
