//--------------------------------------------------------------------------
//
//   Copyright (c) 2002, Colin Granville
//
//   All rights reserved.
//
//   Redistribution and use in source and binary forms, with or
//   without modification, are permitted provided that the following 
//   conditions are met:
//
//      * Redistributions of source code must retain the above copyright 
//        notice, this list of conditions and the following disclaimer.
//
//      * Redistributions in binary form must reproduce the above 
//        copyright notice, this list of conditions and the following 
//        disclaimer in the documentation and/or other materials 
//        provided with the distribution.
//
//      * The name Colin Granville may not be used to endorse or promote 
//        products derived from this software without specific prior 
//        written permission.
//
//   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
//   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
//   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
//   FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
//   COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
//   INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
//   (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
//   SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
//   HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
//   STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
//   OF THE POSSIBILITY OF SUCH DAMAGE.
//
//--------------------------------------------------------------------------

#ifndef DynamicCast_h
#define DynamicCast_h

// basic dynamic cast

// put DECLARE_RTTI() in public section of base class declaration

#define DECLARE_RTTI()\
static const char* typeName();\
virtual void* rtti_isType(void (*t)())

#define PTR_dynamic_cast(ToClass,ptr)  ( (ToClass*)( (ptr)->rtti_isType((void (*)())ToClass::typeName) ) )



// DEFINE_RTTI_BASE or DEFINE_RTTI_DERIVED is required in 
// the definition for each class that uses DECLARE_RTTI

#define DEFINE_RTTI_BASE(Class)\
const char* Class::typeName()                  {return #Class"";}\
void* Class::rtti_isType(void (*t)())          {return t==( (void (*)()) Class::typeName)?this:0;}


#define DEFINE_RTTI_DERIVED(Class,Base)\
const char* Class::typeName(void)                  {return #Class"";}\
void* Class::rtti_isType(void (*t)())          {return t==( (void (*)()) Class::typeName) ?\
                                                            this: Base::rtti_isType(t);}

#endif
