//--------------------------------------------------------------------------
//
//   Copyright (c) 2002, Colin Granville
//
//   All rights reserved.
//
//   Redistribution and use in source and binary forms, with or
//   without modification, are permitted provided that the following 
//   conditions are met:
//
//      * Redistributions of source code must retain the above copyright 
//        notice, this list of conditions and the following disclaimer.
//
//      * Redistributions in binary form must reproduce the above 
//        copyright notice, this list of conditions and the following 
//        disclaimer in the documentation and/or other materials 
//        provided with the distribution.
//
//      * The name Colin Granville may not be used to endorse or promote 
//        products derived from this software without specific prior 
//        written permission.
//
//   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
//   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
//   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
//   FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
//   COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
//   INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
//   (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
//   SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
//   HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
//   STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
//   OF THE POSSIBILITY OF SUCH DAMAGE.
//
//--------------------------------------------------------------------------

#include "Node.h"

DEFINE_RTTI_BASE(Node);

Node::~Node()
{
 if (parent) parent->removeChild(this);
 while (lastChild) delete removeChild(lastChild);
}

//******************************************************************

bool Node::hasChildNodes()                {return 0;}

//******************************************************************

Node* Node::addChild(Node* n)
{
  if (n==0 || !hasChildNodes()) return 0;
  n->parent=this;
  n->next=0;
  n->previous=lastChild;

  if (lastChild) 
     lastChild->next=n;
  else 
     firstChild=n;

  lastChild=n;
  return n;
}

//******************************************************************

Node* Node::removeChild(Node* n)
{
  if (n==0 || n->parent!=this) return 0;

  if (n->previous) 
      n->previous->next=n->next;
  else
      firstChild=n->next;

  if (n->next) 
      n->next->previous=n->previous;
  else
      lastChild=n->previous;

  n->parent=0;
  n->next=0;
  n->previous=0;
  return n;
}
