//--------------------------------------------------------------------------
//
//   Copyright (c) 2002, Colin Granville
//
//   All rights reserved.
//
//   Redistribution and use in source and binary forms, with or
//   without modification, are permitted provided that the following 
//   conditions are met:
//
//      * Redistributions of source code must retain the above copyright 
//        notice, this list of conditions and the following disclaimer.
//
//      * Redistributions in binary form must reproduce the above 
//        copyright notice, this list of conditions and the following 
//        disclaimer in the documentation and/or other materials 
//        provided with the distribution.
//
//      * The name Colin Granville may not be used to endorse or promote 
//        products derived from this software without specific prior 
//        written permission.
//
//   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
//   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
//   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
//   FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
//   COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
//   INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
//   (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
//   SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
//   HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
//   STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
//   OF THE POSSIBILITY OF SUCH DAMAGE.
//
//--------------------------------------------------------------------------

#include "password.h"
#include "GuiWindow.h"
#include "GuiGadget.h"
#include "UserEvents.h"
#include "GuiTargets.h"

class Password
{
  public:
    Password();
    ~Password();
    bool get(string& user,string& owner);

  private:
    GuiWindow        window;
    GuiWritableField password;
    GuiOptionButton  isOwner;
    enum {READY,BUSY,FINISHED,CANCELLED} status;

    GUI_DECLARE_EVENT_TARGETS(Password); 

    GuiToolboxTarget okPressedTarget;
    Claim okPressed(GuiToolboxEvent&,const GuiIdBlock&);

    GuiNullTarget nullEventTarget;
    void nullEvent();
};

//*************************************************************************

Password::Password()
  : window("Password"),
    password(window,1),
    isOwner(window,6),
    okPressedTarget(&window,User_Password,this,(Claim (Password::*)(GuiToolboxEvent&, const GuiIdBlock&))&Password::okPressed),
    status(READY)
{
}

//*************************************************************************

Password::~Password() {}

//*************************************************************************

bool Password::get(string& user,string& owner)
{
  user.clear();
  owner.clear();

  if (status!=READY) return 0;

  window.showCentred();
  password.setValue("");
  isOwner.set(0);
  status=BUSY;
  nullEventTarget(this,(void (Password::*)())&Password::nullEvent);
  while (status==BUSY) GuiRegister::poll();
  window.hide();
  nullEventTarget.destroy();
  user=password.getValue();
  if (isOwner.isOn()) {owner=user;user.clear();}
  if (status==CANCELLED) {status=READY;return 0;}
  status=READY;
  return 1;
}

//*************************************************************************

void Password::nullEvent()
{
  if (status==BUSY && !window.isShowing()) status=CANCELLED;
}

//*************************************************************************

Claim Password::okPressed(GuiToolboxEvent&,const GuiIdBlock&)
{
  status=FINISHED;
  return CLAIM;
}

//*************************************************************************

bool getPassword(string& user,string& owner)
{
  static Password p;
  return p.get(user,owner);
}

