//--------------------------------------------------------------------------
//
//   Copyright (c) 2002, Colin Granville
//
//   All rights reserved.
//
//   Redistribution and use in source and binary forms, with or
//   without modification, are permitted provided that the following 
//   conditions are met:
//
//      * Redistributions of source code must retain the above copyright 
//        notice, this list of conditions and the following disclaimer.
//
//      * Redistributions in binary form must reproduce the above 
//        copyright notice, this list of conditions and the following 
//        disclaimer in the documentation and/or other materials 
//        provided with the distribution.
//
//      * The name Colin Granville may not be used to endorse or promote 
//        products derived from this software without specific prior 
//        written permission.
//
//   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
//   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
//   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
//   FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
//   COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
//   INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
//   (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
//   SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
//   HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
//   STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
//   OF THE POSSIBILITY OF SUCH DAMAGE.
//
//--------------------------------------------------------------------------

#ifndef Outline_h
#define Outline_h

#include "GuiTargets.h"
#include "GuiWindow.h"

class OutlineNode;
class PDFDoc;
class DocView;
class Object;

class Outline
{
  public:
    Outline(PDFDoc&);
    ~Outline();
    bool isOk()               {return pdfDoc != 0;}
    void open(DocView&);
    void close(DocView&);

  private:
    GuiWindow    window;
    DocView*     owner;
    OutlineNode* root;
    PDFDoc*      pdfDoc;

    OutlineNode* foundNode;
    int          foundLevel;
    int          findy;

    int                   height;
    int                   basex;
    int                   basey;
    GuiRedrawWindowBlock* redrawBlock;

    bool                find(OutlineNode* entry,int level=0);
    bool                print(OutlineNode* entry,int level=0);
    static int          countLines(OutlineNode*);

    static OutlineNode* fillTree(OutlineNode* parent,Object*);

    GUI_DECLARE_EVENT_TARGETS(Outline);

    GuiWimpTarget redrawWindowTarget;
    Claim redrawWindow(GuiWimpPollBlock&,const GuiIdBlock&);

    GuiWimpTarget mouseClickTarget;
    Claim mouseClick(GuiWimpPollBlock&,const GuiIdBlock&);
};

#endif
